@@ -20,6 +20,9 @@
  */
 
 const Cairo = imports.gi.cairo;
+const Edit = imports.edit;
+const EvinceView = imports.evinceview;
+const EPUBView = imports.epubview;
 const Gd = imports.gi.Gd;
 const Gdk = imports.gi.Gdk;
 const Gettext = imports.gettext;
@@ -27,6 +30,7 @@ const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
+const LOKView = imports.lokview;
 const _ = imports.gettext.gettext;
 
 const Lang = imports.lang;
@@ -34,9 +38,13 @@ const Mainloop = imports.mainloop;
 
 const Application = imports.application;
 const ErrorBox = imports.errorBox;
+const MainToolbar = imports.mainToolbar;
+const Search = imports.search;
 const WindowMode = imports.windowMode;
 const Utils = imports.utils;
 
+const _ICON_SIZE = 32;
+
 function getController(windowMode) {
     let offsetController;
     let trackerController;
@@ -341,6 +349,10 @@ const ViewContainer = new Lang.Class({
         this.parent({ homogeneous: true,
                       transition_type: Gtk.StackTransitionType.CROSSFADE });
 
+        let actions = this._getDefaultActions();
+        this.actionGroup = new Gio.SimpleActionGroup();
+        Utils.populateActionGroup(this.actionGroup, actions, 'view');
+
         this.view = new Gd.MainView({ shadow_type: Gtk.ShadowType.NONE });
         this.add_named(this.view, 'view');
 
@@ -350,23 +362,23 @@ const ViewContainer = new Lang.Class({
         this._errorBox = new ErrorBox.ErrorBox();
         this.add_named(this._errorBox, 'error');
 
+        this._spinner = new Gtk.Spinner({ width_request: _ICON_SIZE,
+                                          height_request: _ICON_SIZE,
+                                          halign: Gtk.Align.CENTER,
+                                          valign: Gtk.Align.CENTER });
+        this.add_named(this._spinner, 'spinner');
+
         this.show_all();
         this.set_visible_child_full('view', Gtk.StackTransitionType.NONE);
 
         this.view.connect('item-activated',
-                            Lang.bind(this, this._onItemActivated));
+                          Lang.bind(this, this._onItemActivated));
         this.view.connect('selection-mode-request',
-                            Lang.bind(this, this._onSelectionModeRequest));
+                          Lang.bind(this, this._onSelectionModeRequest));
         this.view.connect('view-selection-changed',
-                            Lang.bind(this, this._onViewSelectionChanged));
+                          Lang.bind(this, this._onViewSelectionChanged));
 
-        // connect to settings change for list/grid view
-        let viewSettingsId = Application.application.connect('action-state-changed::view-as',
-            Lang.bind(this, this._updateTypeForSettings));
         this._updateTypeForSettings();
-
-        this._sortSettingsId = Application.application.connect('action-state-changed::sort-by',
-            Lang.bind(this, this._updateSortForSettings));
         this._updateSortForSettings();
 
         // setup selection controller => view
@@ -378,19 +390,6 @@ const ViewContainer = new Lang.Class({
             Lang.bind(this, this._onWindowModeChanged));
         this._onWindowModeChanged();
 
-        let selectAll = Application.application.lookup_action('select-all');
-        let selectAllId = selectAll.connect('activate', Lang.bind(this,
-            function() {
-                Application.selectionController.setSelectionMode(true);
-                this.view.select_all();
-            }));
-
-        let selectNone = Application.application.lookup_action('select-none');
-        let selectNoneId = selectNone.connect('activate', Lang.bind(this,
-            function() {
-                this.view.unselect_all();
-            }));
-
         [ this._offsetController, this._trackerController ] = getController(this._mode);
 
         this._offsetController.connect('item-count-changed', Lang.bind(this,
@@ -410,13 +409,44 @@ const ViewContainer = new Lang.Class({
 
         // this will create the model if we're done querying
         this._onQueryStatusChanged();
+    },
 
-        this.connect('destroy', Lang.bind(this,
-            function() {
-                Application.application.disconnect(viewSettingsId);
-                selectAll.disconnect(selectAllId);
-                selectNone.disconnect(selectNoneId);
-            }));
+    _getDefaultActions: function() {
+        return [
+            { name: 'select-all',
+              callback: Lang.bind(this, this._selectAll),
+              accels: ['<Primary>a'] },
+            { name: 'select-none',
+              callback: Lang.bind(this, this._selectNone) },
+            { settingsKey: 'view-as',
+              stateChanged: Lang.bind(this, this._updateTypeForSettings) },
+            { settingsKey: 'sort-by',
+              stateChanged: Lang.bind(this, this._updateSortForSettings) },
+            { name: 'search-source',
+              parameter_type: 's',
+              state: GLib.Variant.new('s', Search.SearchSourceStock.ALL),
+              stateChanged: Lang.bind(this, this._updateSearchSource),
+              create_hook: Lang.bind(this, this._initSearchSource) },
+            { name: 'search-type',
+              parameter_type: 's',
+              state: GLib.Variant.new('s', Search.SearchTypeStock.ALL),
+              stateChanged: Lang.bind(this, this._updateSearchType),
+              create_hook: Lang.bind(this, this._initSearchType) },
+            { name: 'search-match',
+              parameter_type: 's',
+              state: GLib.Variant.new('s', Search.SearchMatchStock.ALL),
+              stateChanged: Lang.bind(this, this._updateSearchMatch),
+              create_hook: Lang.bind(this, this._initSearchMatch) }
+        ];
+    },
+
+    _selectAll: function() {
+        Application.selectionController.setSelectionMode(true);
+        this.view.select_all();
+    },
+
+    _selectNone: function() {
+        this.view.unselect_all();
     },
 
     _updateTypeForSettings: function() {
@@ -450,7 +480,40 @@ const ViewContainer = new Lang.Class({
         this._model.set_sort_column_id(sortBy, sortType);
     },
 
-    activateResult: function() {
+    _initSearchSource: function(action) {
+        Application.sourceManager.connect('active-changed', Lang.bind(this, function(manager, activeItem) {
+            action.state = GLib.Variant.new('s', activeItem.id);
+        }));
+    },
+
+    _initSearchType: function(action) {
+        Application.searchTypeManager.connect('active-changed', Lang.bind(this, function(manager, activeItem) {
+            action.state = GLib.Variant.new('s', activeItem.id);
+        }));
+    },
+
+    _initSearchMatch: function(action) {
+        Application.searchMatchManager.connect('active-changed', Lang.bind(this, function(manager, activeItem) {
+            action.state = GLib.Variant.new('s', activeItem.id);
+        }));
+    },
+
+    _updateSearchSource: function(action) {
+        let itemId = action.state.get_string()[0];
+        Application.sourceManager.setActiveItemById(itemId);
+    },
+
+    _updateSearchType: function(action) {
+        let itemId = action.state.get_string()[0];
+        Application.searchTypeManager.setActiveItemById(itemId);
+    },
+
+    _updateSearchMatch: function(action) {
+        let itemId = action.state.get_string()[0];
+        Application.searchMatchManager.setActiveItemById(itemId);
+    },
+
+    _activateResult: function() {
         let doc = this._getFirstDocument();
         if (doc)
             Application.documentManager.setActiveItem(doc)
@@ -459,8 +522,8 @@ const ViewContainer = new Lang.Class({
     _getFirstDocument: function() {
         let doc = null;
 
-        let iter = this._model.get_iter_first()[1];
-        if (iter) {
+        let [success, iter] = this._model.get_iter_first();
+        if (success) {
             let id = this._model.get_value(iter, Gd.MainColumns.ID);
             doc = Application.documentManager.getItemById(id);
         }
@@ -561,6 +624,10 @@ const ViewContainer = new Lang.Class({
             // unfreeze selection
             Application.selectionController.freezeSelection(false);
             this._updateSelection();
+
+            // hide the spinner
+            this._spinner.stop();
+            this.set_visible_child_name('view');
         } else {
             // save the last selection
             Application.selectionController.freezeSelection(true);
@@ -568,6 +635,10 @@ const ViewContainer = new Lang.Class({
             // if we're querying, clear the model from the view,
             // so that we don't uselessly refresh the rows
             this.view.set_model(null);
+
+            // kick off the spinner
+            this._spinner.start();
+            this.set_visible_child_name('spinner');
         }
     },
 
@@ -646,5 +717,131 @@ const ViewContainer = new Lang.Class({
             this.view.disconnect(this._edgeHitId);
             this._edgeHitId = 0;
         }
+    },
+
+    createToolbar: function(stack) {
+        let toolbar = new MainToolbar.OverviewToolbar(stack);
+        toolbar.searchbar.connectJS('activate-result',
+                                    Lang.bind(this, this._activateResult));
+        return toolbar;
+    }
+});
+
+const View = new Lang.Class({
+    Name: 'View',
+    Extends: Gtk.Overlay,
+
+    _init: function(window) {
+        this._window = window;
+
+        this.parent();
+
+        this._stack = new Gtk.Stack({ visible: true,
+                                      homogeneous: true,
+                                      transition_type: Gtk.StackTransitionType.CROSSFADE });
+        this.add(this._stack);
+
+        // pack the OSD notification widget
+        this.add_overlay(Application.notificationManager);
+
+        // now create the actual content widgets
+        this._documents = new ViewContainer(WindowMode.WindowMode.DOCUMENTS);
+        let label = Application.application.isBooks ? _('Books') : _("Documents");
+        this._stack.add_titled(this._documents, 'documents', label);
+
+        this._collections = new ViewContainer(WindowMode.WindowMode.COLLECTIONS);
+        this._stack.add_titled(this._collections, 'collections', _("Collections"));
+
+        this._search = new ViewContainer(WindowMode.WindowMode.SEARCH);
+        this._stack.add_named(this._search, 'search');
+
+        this.connect('notify::visible-child',
+                     Lang.bind(this, this._onVisibleChildChanged));
+
+        this.show();
+    },
+
+    _onVisibleChildChanged: function() {
+        let visibleChild = this.visible_child;
+        let windowMode;
+
+        if (visibleChild == this._collections)
+            windowMode = WindowMode.WindowMode.COLLECTIONS;
+        else if (visibleChild == this._documents)
+            windowMode = WindowMode.WindowMode.DOCUMENTS;
+        else
+            return;
+
+        Application.modeController.setWindowMode(windowMode);
+    },
+
+    _clearPreview: function() {
+        if (this._preview) {
+            this._preview.destroy();
+            this._preview = null;
+        }
+    },
+
+    _createPreview: function(mode) {
+        this._clearPreview();
+
+        let constructor;
+        switch (mode) {
+        case WindowMode.WindowMode.PREVIEW_EV:
+            constructor = EvinceView.EvinceView;
+            break;
+        case WindowMode.WindowMode.PREVIEW_LOK:
+            constructor = LOKView.LOKView;
+            break;
+        case WindowMode.WindowMode.PREVIEW_EPUB:
+            constructor = EPUBView.EPUBView;
+            break;
+        case WindowMode.WindowMode.EDIT:
+            constructor = Edit.EditView;
+            break;
+        default:
+            return;
+        }
+
+        this._preview = new constructor(this, this._window);
+        this._stack.add_named(this._preview, 'preview');
+    },
+
+    createToolbar: function() {
+        return this.view.createToolbar(this._stack);
+    },
+
+    set windowMode(mode) {
+        this._clearPreview();
+
+        let visibleChild;
+
+        switch (mode) {
+        case WindowMode.WindowMode.COLLECTIONS:
+            visibleChild = this._collections;
+            break;
+        case WindowMode.WindowMode.DOCUMENTS:
+            visibleChild = this._documents;
+            break;
+        case WindowMode.WindowMode.SEARCH:
+            visibleChild = this._search;
+            break;
+        case WindowMode.WindowMode.PREVIEW_EV:
+        case WindowMode.WindowMode.PREVIEW_LOK:
+        case WindowMode.WindowMode.PREVIEW_EPUB:
+        case WindowMode.WindowMode.EDIT:
+            this._createPreview(mode);
+            visibleChild = this._preview;
+            break;
+        default:
+            return;
+        }
+
+        this._stack.set_visible_child(visibleChild);
+        this._window.insert_action_group('view', visibleChild.actionGroup);
+    },
+
+    get view() {
+        return this._stack.visible_child;
     }
 });
